; Setup Code: Copyright 2005,2006 Justin Weaver.
; This setup code is distributed under the GNU General Public License.
;
;This program is free software; you can redistribute it and/or modify
;it under the terms of the GNU General Public License as published by
;the Free Software Foundation; either version 2 of the License, or
;(at your option) any later version.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#pragma option -v+
#pragma verboselevel 9

#define MyAppName "PiraMod"
#define MyAppDir "PiraMod"
#define MyAppGroupName "PiraMod"
#define MyAppPublisher "HalfNull"
#define MyAppURL "http://piramod.halfnull.com"

#define MyInterface "11200"
#define MyRevision "07"

#define MyVersion MyInterface + "." + MyRevision
#define MyAppVerName "PiraMod" + "_" + MyVersion

[Setup]
; Required
AppName={#MyAppName}
AppVerName={#MyAppVerName}
DefaultDirName={pf}\World of Warcraft

AppPublisher={#MyAppPublisher}
AppPublisherURL={#MyAppURL}
AppSupportURL={#MyAppURL}
AppUpdatesURL={#MyAppURL}

; Compiler-related
OutputDir=.
OutputBaseFilename={#MyAppVerName}
SolidCompression=true
Compression=lzma

; Installer-related
DefaultGroupName={#MyAppGroupName}
UsePreviousAppDir=false
DirExistsWarning=no
DisableProgramGroupPage=true
DisableReadyPage=true
InfoAfterFile=readme.txt
SetupIconFile=PiraMod.ico
UninstallDisplayIcon={app}\{#MyAppDir}\PiraMod.ico
UninstallFilesdir={app}\{#MyAppDir}

[Languages]
Name: english; MessagesFile: compiler:Default.isl

[Dirs]
Name: {app}\Interface; Tasks: 
Name: {app}\Interface\AddOns; Tasks: 
Name: {app}\{#MyAppDir}; Tasks: 
Name: {app}\{#MyAppDir}\Backups; Tasks: 

[Types]
Name: Normal; Description: Choose components; Flags: iscustom

[Files]
Source: 7za.exe; DestDir: {app}; Flags: dontcopy; Components: PiraCore
Source: PiraMod.ico; DestDir: {app}\{#MyAppDir}; Components: PiraCore
Source: PiraMod.iss; DestDir: {app}\{#MyAppDir}; Components: PiraCore
Source: COPYING.txt; DestDir: {app}\{#MyAppDir}; Components: PiraCore
Source: readme.txt; DestDir: {app}\{#MyAppDir}; Components: PiraCore
Source: changelog.txt; DestDir: {app}\{#MyAppDir}; Components: PiraCore
Source: PiraMod\*; DestDir: {app}\Interface\AddOns; Flags: recursesubdirs; Components: PiraCore
Source: PACK_Auctioneer\*; DestDir: {app}\Interface\AddOns; Flags: recursesubdirs; Components: PiraCore
Source: PACK_Bagnon\*; DestDir: {app}\Interface\AddOns; Flags: recursesubdirs; Components: PiraCore
Source: PACK_CT\*; DestDir: {app}\Interface\AddOns; Flags: recursesubdirs; Components: PiraCore
Source: PACK_CTRA\*; DestDir: {app}\Interface\AddOns; Flags: recursesubdirs; Components: PiraCore
Source: PACK_sct\*; DestDir: {app}\Interface\AddOns; Flags: recursesubdirs; Components: PiraCore
Source: PACK_ShardAce\*; DestDir: {app}\Interface\AddOns; Flags: recursesubdirs; Components: PiraCore
Source: PACK_Titan\*; DestDir: {app}\Interface\AddOns; Flags: recursesubdirs; Components: PiraCore
Source: PACK_Zorlen\*; DestDir: {app}\Interface\AddOns; Flags: recursesubdirs; Components: Zorlen
Source: OPT_tradeDispenser\*; DestDir: {app}\Interface\AddOns; Flags: recursesubdirs; Components: tradeDispenser
Source: OPT_Aspected\*; DestDir: {app}\Interface\AddOns; Flags: recursesubdirs; Components: Aspected

[Components]
Name: PiraCore; Description: PiraCore; Types: Normal; Flags: fixed
Name: Zorlen; Description: Zorlen - A Powerful Macro Library.; Flags: disablenouninstallwarning; Languages: 
Name: tradeDispenser; Description: TradeDispnser - Turn yourself into a vending machine.; Flags: disablenouninstallwarning; Languages: 
Name: Aspected; Description: Aspected - Shift/Form/Aspect Bar for Shaman, Priest, and Hunter.; Flags: disablenouninstallwarning; Languages: 

[InstallDelete]
Name: {group}; Type: filesandordirs
Name: {app}\{#MyAppDir}\Version.txt; Type: filesandordirs
Name: {app}\{#MyAppDir}\Documentation; Type: filesandordirs

[Icons]
Name: {group}\My Backups; Filename: {app}\{#MyAppDir}\Backups; Flags: foldershortcut
Name: {group}\My WoW Directory; Filename: {app}
Name: {group}\Launch WoW; Filename: {app}\Launcher.exe
Name: {group}\Uninstall; Filename: {app}\{#MyAppDir}\unins000.exe
Name: {group}\Website; Filename: {app}\{#MyAppDir}\PiraMod.url; IconFilename: {app}\{#MyAppDir}\PiraMod.ico
Name: {group}\ReadMe; Filename: {app}\{#MyAppDir}\readme.txt
Name: {group}\ChangeLog; Filename: {app}\{#MyAppDir}\changelog.txt
Name: {userdesktop}\{#MyAppName}; Filename: {group}; IconFilename: {app}\{#MyAppDir}\PiraMod.ico

[INI]
Filename: {app}\{#MyAppDir}\PiraMod.url; Section: InternetShortcut; Key: URL; String: http://piramod.halfnull.com

[UninstallDelete]
Name: {app}\{#MyAppDir}\PiraMod.url; Type: files

[Code]
var
	ProgressPage: TOutputProgressWizardPage;

procedure PiraMod_Zip(src : string; dst : string; opt : string);
var
	ResultCode : Integer;
	ReturnStatus : Boolean;
	sevenz : string;
begin
	ExtractTemporaryFile('7za.exe');
	sevenz := ExpandConstant('{tmp}\7za.exe');

	ReturnStatus := Exec(sevenz, opt + dst + ' ' + src, '.', SW_HIDE, ewWaitUntilTerminated, ResultCode);

	if (not ReturnStatus) or (ResultCode <> 0) then begin
		MsgBox('ERROR PiraMod_Zip(): [' + opt + ']' + #13 +
			'CurrentDir: ' + GetCurrentDir() + #13 +
			'Executable: '+ sevenz + #13 +
			'Source: ' + src + #13 +
			'Destination: ' + dst + #13 +
			'Error[' + IntToStr(ResultCode) + ']: ' + SysErrorMessage(ResultCode),
			mbInformation,
			MB_OK
		);
		Abort();
	end;
end;

procedure PiraMod_Compress(src : string; dst : string);
begin
	PiraMod_Zip(src, dst, 'a -tzip ');
end;

procedure PiraMod_Expand(src : string; dst : string);
begin
	PiraMod_Zip(src, dst, 'x -o');
end;

procedure InitializeWizard();
begin
	ProgressPage := CreateOutputProgressPage('PiraMod Setup', 'Backing up your current WoW settings.');
end;

procedure CurStepChanged(CurStep: TSetupStep);
var
	timestamp : string;
begin
	if (CurStep = ssInstall) then begin

		if FindWindowByWindowName('World of Warcraft') <> 0 then begin
			MsgBox('You must exit World of Warcraft and/or close any other open windows titled "World of Warcraft" before installing or updating PiraMod.', mbInformation, MB_OK);
			Abort();
		end;

		SetCurrentDir(ExpandConstant('{app}'));

		if FileExists('WoW.exe') = false then begin
			MsgBox('There does not appear to be a WoW installation in "' + ExpandConstant('{app}') + '"', mbInformation, MB_OK);
			Abort();
		end;

		if FileExists('Interface\AddOns\unins000.exe') then begin
			MsgBox('Your current PiraMod installation is too old for me to update. Please uninstall it first before you run this installer again.', mbInformation, MB_OK);
			Abort();
		end;

		if FileExists('Wretched_CT_RaidAssist/unins000.exe') then begin
			MsgBox('Please uninstall Wretched_CT_RaidAssist.', mbInformation, MB_OK);
			Abort();
		end;

		if FileExists('Wretched_DKP_Matrix/unins000.exe') then begin
			MsgBox('WARNING: Wretched_DKP_Matrix appears to be installed. 1) If you proceed with the PiraMod installation, you will need to reinstall Wretched_DKP_Matrix. 2) If you ever uninstall PiraMod, you will need to reinstall Wretched_DKP_Matrix.', mbInformation, MB_OK);
		end;

		ProgressPage.SetProgress(0, 4);

		sleep(500);

		ProgressPage.SetProgress(1, 4);
		ProgressPage.Show;

		timestamp := GetDateTimeString('yyyy_mm_dd_hh_nn_ss', '_', '_');

		{make config backup}
		if DirExists('WTF') then begin
			ProgressPage.SetText('Backing up current WoW configuration...', 'Please wait.');
			PiraMod_Compress('WTF', 'PiraMod\Backups\WTF_' + timestamp + '.zip');
		end;

		ProgressPage.SetProgress(2, 4);

		{make AddOns backup}
		if DirExists('Interface\AddOns') then begin
			ProgressPage.SetText('Backing up current WoW AddOns...', 'Please Wait.');
			PiraMod_Compress('Interface\AddOns', 'PiraMod\Backups\AddOns_' + timestamp + '.zip');
		end;

		ProgressPage.SetProgress(3, 4);

		{clean out any other AddOns and force a refresh of the Blizzard AddOns}
		ProgressPage.SetText('Cleaning AddOns directory...', 'Please Wait.');
		DelTree('Interface\AddOns', True, True, True);

		ProgressPage.SetProgress(4, 4);

		sleep(500);

		ProgressPage.Hide;
	end;
end;
