
RegistryEntry = {  
  id = nil;
  name = nil;
  tooltip = nil;
  buttonFrame = nil;
  updateFunction = nil;
  nextEntry = nil;
  previousEntry = nil;
  origUpdateFunction = nil;
};

  -- constructor
function RegistryEntry:new(obj)
  local o = obj or {};
  setmetatable(o, self);
  self.__index = self;
  return o;
end


ModRegistryClass = {
};

  -- constructor
function ModRegistryClass:new(obj)
  local o = obj or {};
  setmetatable(o, self);
  self.__index = self;
  return o;
end

function ModRegistryClass:GetHead()
  local ret = nil;
  for key,val in self do 
    if (type(val) == "table") then
      if (val.id and (not val.previousEntry)) then
        ButtonHole.util.Debug("got head: "..val.id);
        ret = val.id;
        break;
      end
    end
  end
  return ret;
end

function ModRegistryClass:GetTail() 
  local ret = nil;
  for key,val in self do 
    if (type(val) == "table") then
      if (val.id and (not val.nextEntry)) then
        ButtonHole.util.Debug("got tail: "..val.id);
        ret = val.id;
        break;
      end
    end
  end
  return ret;
end

function ModRegistryClass:AddRegEntry(val)
  if (val ~= nil) then
    local str = val.id.." - "..val.name;
    ButtonHole.util.Debug("addedEntry:"..str);
    local tail = self:GetTail();
    self[val.id] = val;
    if (tail) then
      ButtonHole.util.Debug("addEntry Tail:"..tail);
      self[tail].nextEntry = self[val.id];
      self[val.id].previousEntry = self[tail];
    end
  end
end

function ModRegistryClass:InsertRegEntry(val,prev)
  self[val.id] = val;
  if (prev == nil) then
    -- Inserting to the head of the this
    local currHead = self:GetHead();
    currHead.previousEntry = self[val.id];
    self[val.id].nextEntry = currHead;
  else
    self[val.id].nextEntry = prev.nextEntry;
    prev.nextEntry = self[val.id];
  end
end

function  ModRegistryClass:RemoveRegEntry(id)
  local ret = nil;
  if (self[id] ~= nil) then
    local preve = self[id].previousEntry;
    local nexte = self[id].nextEntry;
    nexte.previousEntry = preve;
    preve.nextEntry = nexte;
    ret = RegistryEntry:new(self[id]);
    self[id] = nil;
  end
  return ret;
end

--[[
Predefined mods to ButtonHole
By joev (joev@jarhedz.com)

$Id: PredefinedMods.lua 15 2005-09-04 10:08:40Z joev $
--]]

PredefinedMods = {};

function PredefinedMods:load()

   PredefinedMods.Atlas = { 
      id="ATLAS", 
      name="Atlas", 
      tooltip = BHSTRINGS.preDefModsTooltip.Atlas, 
      buttonFrame="AtlasButtonFrame", 
      updateFunction="AtlasButton_UpdatePosition"
   };
     
   PredefinedMods.ELL = { 
      id="ELL",
      name="EnhancedLootLink",
      tooltip = "LootLink Enhanced",
      buttonFrame="LootLinkMinimapButton",
      updateFunction="LootLink_Minimap_Position"
   };  
 
   PredefinedMods.CT_RA = { 
      id="CTRA",
      name="CT_RaidAssist",
      tooltip = BHSTRINGS.preDefModsTooltip.CT_RaidAssist,
      buttonFrame="CT_RASets_Button",
      updateFunction="CT_RASets_MoveButton"
   };
   
   PredefinedMods.CT_Mod = { 
      id="CTMOD",
      name="CT_Mod",
      tooltip = BHSTRINGS.preDefModsTooltip.CT_Mod,
      buttonFrame="CT_OptionButton",
      updateFunction="CT_OptionButton_Move"
   };
   
   PredefinedMods.Gatherer = { 
      id="Gatherer",
      name="Gatherer",
      tooltip = BHSTRINGS.preDefModsTooltip.Gatherer,
      buttonFrame="GathererUI_IconFrame",
      updateFunction="GathererUI_MoveButton"
   }; 
   
   PredefinedMods.ItemRack = { 
      id="ItemRack",
      name="ItemRack",
      tooltip = "ItemRack",
      buttonFrame="ItemRack_IconFrame",
      updateFunction="ItemRack_move_icon"
   }; 

   PredefinedMods.FishingBuddy = { 
      id="FishingBuddy",
      name="FishingBuddy",
      tooltip = "FishingBuddy",
      buttonFrame="FishingBuddyMinimapFrame",
      updateFunction="FishingBuddyMinimapButton_MoveButton"
   }; 

end;

--[[
 ButtonHole
    By joev (joev@jarhedz.com)
    
    $Id: ButtonHole.lua 15 2005-09-04 10:08:40Z joev $
--]]

-- Keybindings 
--[[
BINDING_HEADER_BUTTONHOLE               = BHSTRINGS.bindings.title;
BINDING_NAME_BUTTONHOLETOGGLE           = BHSTRINGS.bindings.toggleOnOff;
BINDING_NAME_BUTTONHOLEOPTIONS          = BHSTRINGS.bindings.toggleOptions;
BINDING_NAME_BUTTONHOLEINCREASEOPACITY  = BHSTRINGS.bindings.increaseOpacity;
BINDING_NAME_BUTTONHOLEDECREASEOPACITY  = BHSTRINGS.bindings.decreaseOpacity;
--]]

BH_VERSION = 1.4;

BH_INFO = BHSTRINGS.appName.." "..
                 BH_VERSION.." "..
                 BHSTRINGS.by.." "..
                 BHSTRINGS.author.name;
-- Development...
DEBUG = 0;

BH_COUNTER_MAX = 50;
-- SavedVariables

-- Set up Buttonhole class
ButtonHole = {
  -- Attributes
  modRegistry = nil;  
  player = nil;
  variablesLoaded = false;
  configLoaded = false;
  currentModId = nil;
  showingTooltip = false;
  updateCounter = 0;
  
  -- Constants
  const = {};
  
  -- Utility methods
  util = {};

  -- Application methods
  application = {};
  
  -- UI Management methods
  ui = {};

  -- Event handlers
  events = {};
};

--[[
    ButtonHole constants.
--]]

ButtonHole.const = {
    textColors = {
      RED     = "|cffff0000";
      GREEN   = "|cff00ff00";
      BLUE    = "|cff0000ff";
      MAGENTA = "|cffff00ff";
      YELLOW  = "|cffffff00";
      CYAN    = "|cff00ffff";
      WHITE   = "|cffffffff";
      END     = "|r";
    };
};

--[[
    ButtonHole utility functions.
--]]

ButtonHole.util = {  
      -- Does table contain the given value? 
      TableContainsKey = function(t, val)
        local ret = false;
        if (t and val) then
          for key, value in t do
            if (key == val) then
              ret = true;
              break;
            end
          end
        end
        return ret;
      end;
      
      -- Print Debug info
      Debug = function (s)
       if (DEBUG == 1) then
          DEFAULT_CHAT_FRAME:AddMessage(s, 1, 1, 0)
       end
      end;
    
      -- Print to chat
      Print = function (s)
        DEFAULT_CHAT_FRAME:AddMessage(s)
      end;
    
      PrintTableValues = function(t)
        for key,value in t do 
          ButtonHole.util.Print(value) 
        end    
      end;

      PrintTableKeys = function(t)
        for key,value in t do 
          ButtonHole.util.Print(key) 
        end    
      end;
};

--[[
    ButtonHole ui management functions.
--]]
ButtonHole.ui = {
    -- Swallow a given frame into the hole
    SwallowFrame = function(frameName)
      local frame = getglobal(frameName);
      if (frame ~= nil) then
        frame:ClearAllPoints();
        frame:SetPoint("CENTER",
                       "ButtonHoleFrameBody",
                       "CENTER",
                        0,0);
        frame:Lower();
        frame:Hide();
      end
    end;
    -- Show the given mod entry
    ShowMod = function(regEntry)
      if (ButtonHole.modRegistry[regEntry] ~= nil) then
        local frame = getglobal(ButtonHole.modRegistry[regEntry].buttonFrame);
        if (frame ~= nil) then
           for key,val in ButtonHole.modRegistry do
            if (type(val) == "table" and val.id) then
              ButtonHole.ui.HideMod(val.id);
            end
           end
          frame:Raise();
          frame:Show();
          ButtonHole.ui.ShowTooltip(regEntry);
          ButtonHole.currentModId = regEntry;
          ButtonHoleConfig.visibleMod = regEntry;
          ButtonHole.updateCounter = 0;
        end
      end
    end;
    -- Hide the given mod entry    
    HideMod = function(regEntry)
      if (ButtonHole.modRegistry[regEntry] ~= nil) then
        local frame = getglobal(ButtonHole.modRegistry[regEntry].buttonFrame);
        if (frame ~= nil) then
          if (ButtonHole.showingTooltip) then
            ButtonHole.ui.HideTooltip();
          end
          frame:Lower();
          frame:Hide();
        end
      end
    end;
    -- Show the tooltip
    ShowTooltip = function (regEntry)
      if (ButtonHole.modRegistry[regEntry]) then 
        if (ButtonHole.modRegistry[regEntry].tooltip) then
          GameTooltip:SetOwner(ButtonHoleFrame, "ANCHOR_LEFT");
          GameTooltip:AddLine(ButtonHole.modRegistry[regEntry].name);
          GameTooltip:AddLine(ButtonHole.modRegistry[regEntry].tooltip,.8,.8,.8,1);
          GameTooltip:Show();
          getglobal(ButtonHole.modRegistry[regEntry].buttonFrame):Raise();
          ButtonHole.showingTooltip = true;
        end
      end
    end;
    -- Hide the tooltip
    HideTooltip = function()
      if (ButtonHole.showingTooltip) then
        ButtonHole.showingTooltip = false;
        GameTooltip:Hide();
      end
    end;
    
    Lock = function()
      --ButtonHoleFrameHeader:Hide();
      ButtonHoleFrameHeader:Show();	      
      ButtonHoleConfig.locked = true;
    end;

    Unlock = function()
      ButtonHoleFrameHeader:Show();
      ButtonHoleConfig.locked = false;
    end;
    
    -- Move up the list and show the next button
    ScrollUp = function()
      ButtonHole.util.Debug("scroll up");
      if (ButtonHole.currentModId ~= nil) then
        ButtonHole.ui.HideMod(ButtonHole.currentModId)
        if (ButtonHole.currentModId == ButtonHole.modRegistry:GetTail()) then
          -- We're at the tail - go back to head 
          ButtonHole.currentModId = ButtonHole.modRegistry:GetHead();
        else
          ButtonHole.currentModId = ButtonHole.modRegistry[ButtonHole.currentModId].nextEntry.id;       
        end
        ButtonHole.ui.ShowMod(ButtonHole.currentModId);
      end
    end;
    
    -- Move down the list and show the next button
    ScrollDown = function()
      ButtonHole.util.Debug("scroll down");
      if (ButtonHole.currentModId ~= nil) then
        ButtonHole.ui.HideMod(ButtonHole.currentModId)
        if (ButtonHole.currentModId == ButtonHole.modRegistry:GetHead()) then
          -- We're at the head - go to tail 
          ButtonHole.currentModId = ButtonHole.modRegistry:GetTail();
        else
          ButtonHole.currentModId = ButtonHole.modRegistry[ButtonHole.currentModId].previousEntry.id;       
        end
        ButtonHole.ui.ShowMod(ButtonHole.currentModId);
      end
      
    end;
  };

--[[
    ButtonHole application controller functions.
--]]
ButtonHole.application = {
    -- Register mod with ButtonHole
    RegisterMod = function(modEntry)
      if (modEntry and modEntry.id) then
        if (not ButtonHole.util.TableContainsKey(ButtonHole.modRegistry, modEntry.id)) then
          -- Only add the mod if it actually exists! (Catch the PreDefineds)
          if (getglobal(modEntry.buttonFrame)) then
            local str = modEntry.id..modEntry.name;
            ButtonHole.util.Debug("Entry = "..str);
            ButtonHole.modRegistry:AddRegEntry(modEntry);
          else
            ButtonHole.util.Debug("frame missing: "..modEntry.buttonFrame);
          end
        end
      end
    end;

    -- Show slash command usage
    ShowUsage = function ()
      ButtonHole.util.Print(BH_INFO);
      ButtonHole.util.PrintTableValues(BHSTRINGS.usage);
    end;

    -- Show the application credits.
    ShowCredits = function()
      ButtonHole.util.Print(BHSTRINGS.appName.." v"..BH_VERSION.." - "..BHSTRINGS.creditsInfo.."\n\n");

      for key,value in BHSTRINGS.credits do
          ButtonHole.util.Print(ButtonHole.const.textColors.CYAN..value.task);
          if (value.alias == "") then
              ButtonHole.util.Print(ButtonHole.const.textColors.YELLOW..value.name.."\n\n");
          else
              ButtonHole.util.Print(ButtonHole.const.textColors.YELLOW..value.name..BHSTRINGS.aka..value.alias.."\n\n");
          end
      end
    end;

    -- Setup config variables and initialise the application.
    ConfigInit = function()
      if (not ButtonHole.configLoaded) then
        if (not ButtonHoleConfig) then
            ButtonHoleConfig = {};
            ButtonHoleConfig.on = true;
            ButtonHoleConfig.visibleMod = nil;            
            ButtonHoleConfig.locked = false;
            ButtonHoleConfig.version = tonumber(BH_VERSION);
        else
          if (not ButtonHoleConfig.version) then
            ButtonHoleConfig.version = 1;
          end
          if (ButtonHoleConfig.version < tonumber(BH_VERSION)) then
            ButtonHoleConfig.version = tonumber(BH_VERSION);
          end
        end
        ButtonHole.configLoaded = true;
        ButtonHole.util.Debug("Finished Config Loading");

        ButtonHole.application.LoadPredefinedMods();
        
        ButtonHole.application.SetOn(ButtonHoleConfig.on);

        --ButtonHole.util.Print(BH_INFO..BHSTRINGS.initialized);
      end
    end;

    -- Toggle the application on/off  
    SetOn = function(val)
      ButtonHoleConfig.on = val;
      ButtonHole.application.Setup();
    end;

    -- Setup the application
    Setup = function()    
      if (ButtonHole.application.IsOn()) then
        -- Swallow the Mods in the registry into the hole... We're managing them now.
        for key,val in ButtonHole.modRegistry do
          if (type(val) == "table" and val.id) then
            -- Swallow the frame
            ButtonHole.ui.SwallowFrame(val.buttonFrame);
            -- Hide it...
            getglobal(val.buttonFrame):Hide();
            -- Disable mods "move minimap button" method
            local origFunc = getglobal(val.updateFunction);
            if (origFunc == nil) then
              ButtonHole.util.Debug("origFunc is nil... what's up with that?");
            end
            ButtonHole.util.Debug("attempting override of: "..val.updateFunction);
            ButtonHole.util.Debug("trapping orig...");
            ButtonHole.modRegistry[val.id].origUpdateFunction = origFunc;
            ButtonHole.util.Debug("overriding with setglobal...");
            setglobal(val.updateFunction, function (...) return (nil) end);
            ButtonHole.util.Debug("Orig func overriden");
          end  
        end
        if ((ButtonHoleConfig.visibleMod ~= nil) and (ButtonHole.modRegistry[regEntry] ~= nil)) then
          ButtonHole.ui.ShowMod(ButtonHoleConfig.visibleMod)
        else 
          ButtonHole.ui.ShowMod(ButtonHole.modRegistry:GetHead());
        end
        if (ButtonHoleConfig.locked) then
          --ButtonHole.util.Print("Locked position");
          ButtonHole.ui.Lock();
        else
          --ButtonHole.util.Print("Unlocked position");
          ButtonHole.ui.Unlock();
        end
        UIDropDownMenu_Initialize(ButtonHoleDropdownMenu, ButtonHole.application.DropDownInitialize, "MENU");
        UIDropDownMenu_SetAnchor(0, 0, ButtonHoleDropdownMenu, "TOPRIGHT", "ButtonHoleFrameHeader", "TOPLEFT")
 
      end
    end;
    
    -- Is the application on?
    IsOn = function()
      return ButtonHoleConfig.on;
    end;
    
    IsLocked = function()
      return ButtonHoleConfig.locked;
    end;
    
    -- Handle slash commands
    SlashHandler = function(msg)
        if msg == nil or msg == "" then
            msg = "help";
        end
        msg = strlower(msg);
        local args = {n=0}
        local function helper(word) table.insert(args, word) end
        string.gsub(msg, "[_%w]+", helper);
        if args[1] == 'on'  then
            ButtonHole.application.SetOn(true);
	    ButtonHole.util.Print(ButtonHole.const.textColors.CYAN..BHSTRINGS.appName..ButtonHole.const.textColors.YELLOW.." :: "..ButtonHole.const.textColors.WHITE..BHSTRINGS.isNow..ButtonHole.const.textColors.GREEN..BHSTRINGS.on);
        elseif args[1] == 'off' then
            ButtonHole.application.SetOn(false);
	    ButtonHole.util.Print(ButtonHole.const.textColors.CYAN..BHSTRINGS.appName..ButtonHole.const.textColors.YELLOW.." :: "..ButtonHole.const.textColors.WHITE..BHSTRINGS.isNow..ButtonHole.const.textColors.RED..BHSTRINGS.off);
        elseif args[1] == 'lock' then
            ButtonHole.ui.Lock();
        elseif args[1] == 'unlock' then
            ButtonHole.ui.Unlock();
        elseif args[1] == 'credits' then
            ButtonHole.application.ShowCredits();
        else
            ButtonHole.application.ShowUsage();
        end
    end;
    
    LoadPredefinedMods = function()

      ButtonHole.util.Debug("loading predefs");
      PredefinedMods:load();

      for key,val in PredefinedMods do
        if (type(val) == "table" and val.id) then      
          ButtonHole.util.Debug("adding: "..val.id);
          ButtonHole.application.RegisterMod(val);
        end
      end      
    end;
    
    ToggleLock = function()
      if (ButtonHoleConfig.locked) then
        ButtonHole.ui.Unlock();
      else
        ButtonHole.ui.Lock();
      end
    end;

    DropDownInitialize = function()
      local info;
        info = {};
        info.text = BHSTRINGS.appName.." "..BH_VERSION;
        info.notClickable = 1;
        info.isTitle = 1;
        UIDropDownMenu_AddButton(info);

        info = {};
        info.disabled = 1;
        UIDropDownMenu_AddButton(info);

        info = {};
        info.text = BHSTRINGS.dropdown.lock;
        info.func = ButtonHole.application.ToggleLock;
        info.checked = ButtonHole.application.IsLocked();
        UIDropDownMenu_AddButton(info);

        info = {};
        info.text = BHSTRINGS.dropdown.options;
        info.disabled = 1;
        --info.func = AcceptBattlefieldPort;
        --info.notCheckable = 1;
        UIDropDownMenu_AddButton(info);
    end;
  };
--[[
    ButtonHole event handler functions.
--]]
ButtonHole.events = {
    
    -- Load event handler.
    OnLoad = function ()
        this:RegisterEvent("VARIABLES_LOADED");
        this:RegisterEvent("PLAYER_ENTERING_WORLD");
    
        -- Slash Commands
        SlashCmdList["BH"] = ButtonHole.application.SlashHandler;
        SLASH_BH1 = "/buttonhole";
        SLASH_BH2 = "/bh";
    
        ButtonHole.modRegistry = ModRegistryClass:new();
        
        
    end;
    
    -- General event handler.
    OnEvent = function()
      if (event == "VARIABLES_LOADED") then
          ButtonHole.variablesLoaded = true;
          ButtonHole.util.Debug("Variables Loaded");
          if (ButtonHole.player) then
              ButtonHole.application.ConfigInit();
          end
          -- Add myAddOns support
          if myAddOnsList then
              myAddOnsList.ButtonHole = {name = "ButtonHole", description = BHSTRINGS.myAddonsDescription, version = BH_VERSION, category = MYADDONS_CATEGORY_OTHER, frame = "ButtonHoleFrame", optionsframe = ""};
          end
       elseif( event == "PLAYER_ENTERING_WORLD" and not ButtonHole.player) then
        local name = UnitName("player");
        if( name and name ~= UNKNOWNOBJECT ) then
          ButtonHole.player = name;
          ButtonHole.util.Debug("Unit Name: player = "..ButtonHole.player);
          if (ButtonHole.variablesLoaded) then
            ButtonHole.application.ConfigInit();
          end
        end
      end        
    end;
    
    -- Mousewheel event handler
    OnMouseWheel = function(value)
      if (ButtonHole.application.IsOn()) then
        if ( value > 0 ) then 
          ButtonHole.ui.ScrollUp();
        elseif ( value < 0 ) then 
          ButtonHole.ui.ScrollDown();
        end
      end
    end;
    
    OnRightClick = function()
        if (ButtonHole.application.IsOn()) then
	  ButtonHole.ui.ScrollDown();   
	end
    end;

    OnUpdate = function()
      if (ButtonHole.updateCounter > BH_COUNTER_MAX) then
        -- Time to do something.
        ButtonHole.updateCounter = 0;
        if (MouseIsOver(ButtonHoleFrame) and not GameTooltip:IsShown()) then
          ButtonHole.ui.ShowTooltip(ButtonHole.currentModId);
           for key,val in ButtonHole.modRegistry do
            if (type(val) == "table" and val.id) then
              --Correct some errors that can occur
              if (getglobal(val.buttonFrame):IsVisible() and (val.id ~= ButtonHole.currentModId)) then
                -- You're not supposed to be here... back in the Hole!
                ButtonHole.ui.HideMod(val.id);
              end
              if (not getglobal(ButtonHole.modRegistry[ButtonHole.currentModId].buttonFrame):IsVisible()) then
                -- It rubs the lotion on it's skin or else it gets the hose again...
                ButtonHole.ui.ShowMod(ButtonHole.currentModId);
              end
            end
          end
        else
          if (not MouseIsOver(ButtonHoleFrame) and ButtonHole.showingTooltip) then
            ButtonHole.ui.HideTooltip();
          end
        end
      end
      ButtonHole.updateCounter = ButtonHole.updateCounter + 1;
    end;
    
    header = {
      OnEnter = function()
        GameTooltip:SetOwner(ButtonHoleFrameHeader, "ANCHOR_LEFT");
        GameTooltip:SetText(BH_INFO, 255/255, 209/255, 0/255);
        GameTooltip:AddLine(BHSTRINGS.header.tooltip, 1.00, 1.00, 1.00);  
        GameTooltip:Show();
      end;

      OnLeave = function()
        GameTooltip:Hide();
      end;
    };
};
