--== RecipeBookSearch.lua ==--
-- Contains functions necessary for the RecipeBook Search Frame
RB_SearchResults = {};

function RecipeBookSearch_OnLoad()
end

function RecipeBookSearch_OnEvent()
end

function RecipeBookSearch_DoSearch()
	RecipeBook_Debug("Searching on: "..RecipeBook_SearchFor:GetText());
	for n = 1, table.getn(RB_SearchResults),1 do
		table.remove(RB_SearchResults);
	end
	RecipeBook_SearchResults_ScrollFrame:Hide();
	if RecipeBookSearch_CBX_SearchItems:GetChecked() then RecipeBookSearch_MatchItems(RecipeBook_SearchFor:GetText()) end;
	if RecipeBookSearch_CBX_SearchMats:GetChecked() then RecipeBookSearch_MatchMaterials(RecipeBook_SearchFor:GetText()) end;
	table.sort(RB_SearchResults, function(a,b) return a[1]<b[1] end);
	RecipeBook_Debug(table.getn(RB_SearchResults));
	if table.getn(RB_SearchResults) < 1 then table.insert(RB_SearchResults, {RECIPEBOOK_RED..RECIPEBOOK_ERR_NOSEARCHRESULT..RECIPEBOOK_END, ""}) end;
--	FauxScrollFrame_SetOffset(RecipeBook_SearchResults_ScrollFrame, 0);
	RecipeBook_SearchFor:HighlightText(0);
 	RecipeBook_SearchResults_ScrollFrame:Show();
end

function RecipeBookSearch_Button_OnClick(button)
	local text = getglobal("RecipeBook_SearchFrameEntry"..this:GetID().."_TextItem"):GetText();
	local index = FauxScrollFrame_GetOffset(RecipeBook_SearchResults_ScrollFrame) + this:GetID();
	if ( button == "LeftButton" ) then
		local link, id, skill = unpack(RB_SearchResults[index][3]);
		if link == nil then
			if ( ChatFrameEditBox:IsVisible() ) then
				ChatFrameEditBox:Insert(text);
			end
			return;
		end
		if IsShiftKeyDown() then 
			if ( ChatFrameEditBox:IsVisible() ) then
				ChatFrameEditBox:Insert(link);
			end
		elseif IsAltKeyDown() then
			local data = {};
			table.foreach(RecipeBookMasterList["Tradeskills"][skill][text]["Materials"], function(a,b) table.insert(data, RecipeBookMasterList["Links"][a].." x"..b) end);
			if ( ChatFrameEditBox:IsVisible() ) then
				ChatFrameEditBox:Insert(table.concat(data, ", "));
			else
				RecipeBook_Print(RECIPEBOOK_TXT_SUPPLIES..link..": "..table.concat(data, ", "), -1, nil, 0.25,0.5,1);
			end
		else
			SetItemRef(id,link);
		end
	else
		RecipeBookSearchTooltip:ClearLines();
		RecipeBookSearchTooltip:SetOwner(this, "ANCHOR_CURSOR");
		RecipeBookSearchTooltip:AddLine(text, 1, 1, 0);
		for n = 1, table.getn(RB_SearchResults[index][4]), 1 do
			RecipeBookSearchTooltip:AddLine(RB_SearchResults[index][4][n]);
		end
		RecipeBookSearchTooltip:Show();
	end
end

function RecipeBookSearch_ResultsScrollBar_Update()
	RecipeBook_Debug("Entering ScrollBar_Update");
	local length = table.getn(RB_SearchResults);
	RecipeBook_Debug("Length: "..length);
	local line, index, button, text, item, flavor; 
	local offset = FauxScrollFrame_GetOffset(RecipeBook_SearchResults_ScrollFrame);
	RecipeBook_Debug("Offset: "..offset);
	for line = 1,10,1 do 
		index = offset + line;
		item, flavor = nil, nil;
		if RB_SearchResults[index] ~= nil then
			item = RB_SearchResults[index][1];
			flavor = RB_SearchResults[index][2];
		end
		textItem = getglobal("RecipeBook_SearchFrameEntry"..line.."_TextItem");
		textItem:SetText(item);
		textFlavor = getglobal("RecipeBook_SearchFrameEntry"..line.."_TextFlavor");
		textFlavor:SetText(flavor);
		button = getglobal("RecipeBook_SearchFrameEntry"..line);
		button:SetID(line);
		if index > length then 
			button:Hide();
		else
			button:Show();
		end
	end
	FauxScrollFrame_Update(RecipeBook_SearchResults_ScrollFrame,length+1,10,40);
end

function RecipeBookSearch_MatchItems(match)
	local function parseTradeskills(skill,items)
		local function makeTooltip(recipe, id)
			local function addName(num, who)
				who = string.gsub(who, ",", "");
				table.insert(knownlines, who);
			end
			-- {item name, flavor text, {Left-click tooltip lines}, {Right-click tooltip lines}}
			local altsknow, altshaveskill, _, _ = RecipeBook_MatchRecipeData(recipe,recipe, skill, 0);
			if altsknow == "" then altsknow = {} 
			else
				altsknow = RecipeBook_Split(altsknow);
			end;
			if altshaveskill == "" then 
				altshaveskill = {};
			else
				altshaveskill = RecipeBook_Split(altshaveskill);
			end;
			local text = string.format(RECIPEBOOK_SEARCHTEXT_ITEMSTEMPLATE, table.getn(altsknow), table.getn(altshaveskill), skill);
			if table.getn(altsknow) > 0 then
				knownlines = {"Known by:"};
				table.foreachi(altsknow, addName);
				if table.getn(altshaveskill) > 0 then 
					table.insert(knownlines, " ");
					table.insert(knownlines, "Have "..skill..":");
					table.foreachi(altshaveskill, addName);
				else
				end
			else
				knownlines = {"Known by no-one"};
			end
			table.insert(RB_SearchResults, {recipe, text, {RecipeBookMasterList["Links"][id], id, skill}, knownlines});
		end
		
		table.foreach(items, function(a,b) if string.find(string.lower(a), string.lower(match)) then makeTooltip(a, b["ID"]); end end);
--		table.insert(RB_SearchResults, {a, skill}) 
	end
	
	table.foreach(RecipeBookMasterList["Tradeskills"], parseTradeskills);
end


function RecipeBookSearch_MatchMaterials(match)
-- 	RecipeBook_Print("Searching for items that use materials matching '"..match.."'...", 0);
	local matchlist = {};
	local itemlist = {};
	local skill;
	local Realm, Faction, OtherFaction = RecipeBook_GetGlobals();
	
	local function parseLinks(id, link)
		_,_, text = string.find(link[1], "%[(.*)%]");
		_,_, text2 = string.find(link[3], "%[(.*)%]");
		found = nil;
		for n = 1, table.getn(RB_SearchResults), 1 do
			if RB_SearchResults[n][1] == text then 
				if string.find(RB_SearchResults[n][2], RECIPEBOOK_SEARCHTEXT_MATSPREFIX) then
					if string.sub(RB_SearchResults[n][2], -3) ~= "..." then
						--Already set up as long as it can get.
					elseif string.len(RB_SearchResults[n][2]..", "..text2) < 60 then 
						RB_SearchResults[n][2] = RB_SearchResults[n][2]..", "..text2;
					else
						RB_SearchResults[n][2] = RB_SearchResults[n][2].."...";
					end
				else
					-- Do not add another instance of this item.
				end
				found = true;
			end
		end
		
		if not found then
			local altsknow = RecipeBook_MatchRecipeData(text, text, link[4], 0);
			if altsknow == "" then altsknow = {} 
			else
				altsknow = RecipeBook_Split(altsknow);
			end;
			for n = 1, table.getn(altsknow), 1 do
				altsknow[n] = string.gsub(altsknow[n], ",", "");
				_,_,name = string.find(altsknow[n], "|c%x%x%x%x%x%x%x%x(%.*)|r");
				if RecipeBookData[Realm][Faction] ~= nil then 
					if RecipeBookData[Realm][Faction]["Personal"] ~= nil and RecipeBookData[Realm][Faction]["Personal"][name] ~= nil then 
						altsknow[n] = altsknow[n].." ("..RecipeBookData[Realm][Faction]["Personal"][name][link[4]][text]..")";
					elseif RecipeBookData[Realm][Faction]["Shared"] ~= nil and RecipeBookData[Realm][Faction]["Shared"] ~= nil and RecipeBookData[Realm][Faction]["Shared"][name] ~= nil then
						altsknow[n] = altsknow[n].." ("..RecipeBookData[Realm][Faction]["Shared"][name][link[4]][text]..")";
					end
				end
				if RecipeBookData[Realm][OtherFaction] ~= nil then
					if RecipeBookData[Realm][OtherFaction]["Personal"] ~= nil and RecipeBookData[Realm][OtherFaction]["Personal"][name] ~= nil then
						altsknow[n] = altsknow[n].." ("..RecipeBookData[Realm][OtherFaction]["Personal"][name][link[4]][text]..")";
					elseif RecipeBookData[Realm][OtherFaction]["Shared"] ~= nil and RecipeBookData[Realm][OtherFaction]["Shared"][name] ~= nil then
						altsknow[n] = altsknow[n].." ("..RecipeBookData[Realm][OtherFaction]["Shared"][name][link[4]][text]..")";
					end
				end
			end
			if table.getn(altsknow) > 0 then
				knownlines = {"Can be made by:"};
				table.foreachi(altsknow, function(a,b) table.insert(knownlines, b) end );
			else
				knownlines = {"Known by no-one"};
			end

 			-- {item name, flavor text, {Left-click tooltip lines}, {Right-click tooltip lines}}
			table.insert(RB_SearchResults, {text, RECIPEBOOK_SEARCHTEXT_MATSPREFIX..text2, {link[1], link[2], link[4]}, knownlines});
		end
	end
	
	local function checkMaterials(what, contents)
		RecipeBook_Debug(skill);
		table.foreach(matchlist, function(a,b) if contents["Materials"][b] ~= nil then table.insert(itemlist, {RecipeBookMasterList["Links"][contents["ID"]],contents["ID"], RecipeBookMasterList["Links"][b], skill}) end end);
	end
	
--	if table.getn(matchlist) < 1 then return RecipeBook_Print("No material matches found for '"..match.."'.", -1) 
--	else RecipeBook_Print(table.getn(matchlist).." match(es) found for '"..match.."'. Now searching for recipes using those materials.");
--	end
	

	table.foreach(RecipeBookMasterList["Links"], function(a,b) if string.find(string.lower(b), string.lower(match)) then table.insert(matchlist, a) end end);
	table.foreach(RecipeBookMasterList["Tradeskills"], function(a,b) skill = a; table.foreach(b, checkMaterials) end);
	table.foreachi(itemlist, parseLinks);
end